package com.jiuqi.util;

import java.util.Arrays;

public final class IntList
{
    protected int size;
    protected int[] data;
    
    public IntList() {
    }
    
    public IntList(final int capacity) {
        this.setCapacity(capacity);
    }
    
    public int size() {
        return this.size;
    }
    
    public void reset() {
        this.size = 0;
    }
    
    public void clear() {
        this.size = 0;
        this.data = null;
    }
    
    public int get(final int index) {
        return this.data[index];
    }
    
    public void set(final int index, final int value) {
        this.data[index] = value;
    }
    
    public int add(final int value) {
        final int result = this.size;
        this.setSize(this.size + 1);
        this.data[result] = value;
        return result;
    }
    
    public void insert(final int index, final int value) {
        this.setSize(this.size + 1);
        for (int i = this.size - 1; i > index; --i) {
            this.data[i] = this.data[i - 1];
        }
        this.data[index] = value;
    }
    
    public void remove(final int index) {
        for (int i = index; i < this.size - 1; ++i) {
            this.data[i] = this.data[i + 1];
        }
        this.setSize(this.size - 1);
    }
    
    public void setSize(final int value) {
        if (value <= 0) {
            this.removeAll();
        }
        else if (value != this.size) {
            if (this.data == null || this.data.length < value) {
                this.data = resize(this.data, value);
            }
            else if (value > this.size) {
                for (int i = this.size; i < value; ++i) {
                    this.data[i] = 0;
                }
            }
            this.size = value;
        }
    }
    
    public void setCapacity(final int value) {
        if (value >= this.size) {
            this.data = resize(this.data, value);
        }
    }
    
    public void removeAll() {
        this.size = 0;
    }
    
    public void setAll(final int value) {
        if (this.data != null) {
            Arrays.fill(this.data, value);
        }
    }
    
    public void plus(final int index, final int value) {
        final int[] data = this.data;
        data[index] += value;
    }
    
    public void setSmaller(final int index, final int value) {
        if (value < this.data[index]) {
            this.data[index] = value;
        }
    }
    
    public void setLarger(final int index, final int value) {
        if (value > this.data[index]) {
            this.data[index] = value;
        }
    }
    
    public int getTotal() {
        int value = 0;
        for (int i = 0; i < this.size; ++i) {
            value += this.data[i];
        }
        return value;
    }
    
    public int getTotal(final int fromIndex, final int toIndex) {
        int value = 0;
        for (int i = fromIndex; i <= toIndex; ++i) {
            value += this.data[i];
        }
        return value;
    }
    
    public boolean isEmpty() {
        return this.size == 0;
    }
    
    public void assign(final IntList source) {
        this.setSize(source.size);
        copy(source.data, this.data, this.size);
    }
    
    public int indexOf(final int value) {
        for (int i = 0; i < this.size; ++i) {
            if (this.data[i] == value) {
                return i;
            }
        }
        return -1;
    }
    
    public void removeValue(final int value) {
        final int index = this.indexOf(value);
        if (index >= 0) {
            this.remove(index);
        }
    }
    
    public int find(final int value) {
        int sb = 0;
        int se = this.size - 1;
        int flag = -1;
        while (sb <= se) {
            final int sp = sb + se >> 1;
            flag = value - this.data[sp];
            if (flag > 0) {
                sb = sp + 1;
            }
            else {
                if (flag >= 0) {
                    sb = sp;
                    break;
                }
                se = sp - 1;
            }
        }
        return (flag == 0) ? sb : (-sb - 1);
    }
    
    public int findFirst(final int value) {
        int sb = 0;
        int se = this.size - 1;
        int flag = -1;
        while (sb <= se) {
            final int sp = sb + se >> 1;
            flag = value - this.data[sp];
            if (flag > 0) {
                sb = sp + 1;
            }
            else if (flag < 0) {
                se = sp - 1;
            }
            else {
                if (sb == (se = sp)) {
                    break;
                }
                continue;
            }
        }
        return (flag == 0) ? sb : (-sb - 1);
    }
    
    public int findLast(final int value) {
        int sb = 0;
        int se = this.size - 1;
        int flag = -1;
        while (sb <= se) {
            final int sp = sb + se + 1 >> 1;
            flag = value - this.data[sp];
            if (flag > 0) {
                sb = sp + 1;
            }
            else if (flag < 0) {
                se = sp - 1;
            }
            else {
                if (se == (sb = sp)) {
                    break;
                }
                continue;
            }
        }
        return (flag == 0) ? sb : (-sb - 1);
    }
    
    public void sortAdd(final int value) {
        final int index = this.find(value);
        if (index < 0) {
            this.insert(-index - 1, value);
        }
    }
    
    public void sortAddLast(final int value) {
        final int index = this.findLast(value);
        if (index < 0) {
            this.insert(-index - 1, value);
        }
        else {
            this.insert(index + 1, value);
        }
    }
    
    public void exchange(final int index1, final int index2) {
        if (index1 != index2) {
            final int t = this.data[index1];
            this.data[index1] = this.data[index2];
            this.data[index2] = t;
        }
    }
    
    public void move(final int index1, final int index2) {
        if (index1 > index2) {
            final int t = this.data[index1];
            for (int i = index1; i > index2; --i) {
                this.data[i] = this.data[i - 1];
            }
            this.data[index2] = t;
        }
        else if (index1 < index2) {
            final int t = this.data[index1];
            for (int i = index1; i < index2; ++i) {
                this.data[i] = this.data[i + 1];
            }
            this.data[index2] = t;
        }
    }
    
    public int getFirst() {
        return this.data[0];
    }
    
    public int getLast() {
        return this.data[this.size - 1];
    }
    
    public void push(final int value) {
        this.add(value);
    }
    
    public int pop() {
        return this.data[this.size--];
    }
    
    public int[] toArray() {
        if (this.size == 0) {
            return null;
        }
        final int[] result = new int[this.size];
        copy(this.data, result);
        return result;
    }
    
    public int toArray(final int[] data) {
        copy(this.data, data, this.size);
        return this.size;
    }
    
    public static int[] resize(final int[] data, int newSize) {
        if (newSize < 256) {
            newSize = (newSize + 63 & 0xFFFFFFC0);
        }
        else if (newSize < 4096) {
            newSize = (newSize + 255 & 0xFFFFFF00);
        }
        else {
            newSize = (newSize + 4095 & 0xFFFFF000);
        }
        final int[] newData = new int[newSize];
        if (data != null) {
            copy(data, newData);
        }
        return newData;
    }
    
    public static void copy(final int[] src, final int[] dst) {
        if (src == null || dst == null) {
            return;
        }
        int count = src.length;
        if (count > dst.length) {
            count = dst.length;
        }
        for (int i = 0; i < count; ++i) {
            dst[i] = src[i];
        }
    }
    
    public static void copy(final int[] src, final int[] dst, int count) {
        if (src == null || dst == null) {
            return;
        }
        if (count > src.length) {
            count = src.length;
        }
        if (count > dst.length) {
            count = dst.length;
        }
        for (int i = 0; i < count; ++i) {
            dst[i] = src[i];
        }
    }
    
    @Override
    public String toString() {
        final StringBuffer sb = new StringBuffer(this.size() * 4);
        sb.append('(');
        for (int i = 0; i < this.size; ++i) {
            sb.append(this.data[i]);
            if (i < this.size - 1) {
                sb.append(',');
            }
        }
        sb.append(")");
        return sb.toString();
    }
}
